#include "stdafx.h"
#include "WallpaperQ.h"
#include "WallpaperQDlg.h"
#include "registry.h"
#include <time.h>
#include <stdlib.h>
#include "OptionsDialog.h"
#include "utility.h"
#include "AutoUpdater.h"
#include "MainHelperQ.h"
#include ".\wallpaperqdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Helper m_Helper;

extern "C"
{
	typedef BOOL (*LPSetHook)(HWND hWnd);
	typedef BOOL (*LPClearHook)(HWND hWnd);
}

#define NETWORK_NEIGHBOUR   _T("Network Neighborhood")

bool ParsePath(CString sPath, int nLevel, CString *const psDirName)
{
	ASSERT(psDirName);
	//find first match
	int nStart = 0;									//Location in sPath to search from
	while (nLevel)
	{
		nStart = sPath.Find(_T('\\'), nStart);
		if (nStart < 0)
			return false;
		nLevel--;
		nStart++;
	}
	//We now have the start point to find the end.
	int nEnd = sPath.Find(_T('\\'), nStart);
	if (nEnd < 0)
		*psDirName = sPath.Mid(nStart);
	else
		*psDirName = sPath.Mid(nStart, nEnd - nStart);
	return true;
}

TCHAR* MakeObjectDynamic(LPTSTR szData)
{
	//Assume a NULL = empty string
	TCHAR* szRet = NULL;
	int nLength = 0;
	if (szData)
		nLength = _tcslen(szData) + 1;
	if (nLength > 0)
	{
		szRet = new TCHAR[nLength];
		ASSERT(szRet);
		_tcscpy(szRet, szData);
	}
	return szRet;
}

/*************************/
/* CWallpaperQDlg dialog */
/*************************/
CWallpaperQDlg::CWallpaperQDlg() : CDialog(CWallpaperQDlg::IDD), uiTimer(0)
{
	m_uXPos = 0;
	m_uYPos = 0;
	m_nTransparency = 0;
	m_nBackgroundTransparency = 0;
	m_enableCalendar = 0;

	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_buildDesktopImage = new CBuildDesktopImage(this->m_hWnd);
	clock = new CClock();
	// Get a pointer to the IActiveDesktop interface.
	// We use this to set the wallpaper when not using .BMP files.
	pIActiveDesktop = NULL;
	HRESULT hr = CoCreateInstance(CLSID_ActiveDesktop, NULL, CLSCTX_INPROC_SERVER, IID_IActiveDesktop, (void**)&pIActiveDesktop);

	if (FAILED(hr))
		pIActiveDesktop = NULL;

	bIconsShown = TRUE;
}

CWallpaperQDlg::~CWallpaperQDlg()
{
	if (hAOT != NULL)
	{
		LPClearHook pClearHook = (LPSetHook)GetProcAddress(hAOT, "ClearHook");
		if (pClearHook != NULL)
		{
			(pClearHook)(this->m_hWnd);
		}
		FreeLibrary(hAOT);
		hAOT = NULL;
	}
	delete m_buildDesktopImage;
	delete clock;
	// Remove the system tray icon.
	Shell_NotifyIcon(NIM_DELETE, &niData);

	// Release the IActiveDesktop interface.
	if (pIActiveDesktop != NULL)
	{
		pIActiveDesktop->Release();
		pIActiveDesktop = NULL;
	}
}

void CWallpaperQDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BROWSELIST, m_lbBrowseList);
	DDX_Control(pDX, IDC_PREVIEW, m_picPreview);
	DDX_Control(pDX, IDC_PREVIEW2, m_picPreview2);
	DDX_Control(pDX, IDC_POSITION, m_comboPosition);
	DDX_Control(pDX, IDC_QLIST, m_lbQList);
	DDX_Control(pDX, IDC_TYPE, m_stType);
	DDX_Control(pDX, IDC_HEIGHT, m_stHeight);
	DDX_Control(pDX, IDC_WIDTH, m_stWidth);
	DDX_Control(pDX, IDC_TYPE2, m_stType2);
	DDX_Control(pDX, IDC_HEIGHT2, m_stHeight2);
	DDX_Control(pDX, IDC_WIDTH2, m_stWidth2);
	DDX_Control(pDX, IDC_RESOLUTION, m_stResolution);
	DDX_Control(pDX, IDC_NUMCOLORS, m_stColors);
	DDX_Control(pDX, IDC_QLIST, m_lbQList);
	DDX_Control(pDX, IDC_RANDOMIZE, m_chkRandomize);
	DDX_Text(pDX, IDC_NBROWS, m_nbRows);
	DDX_Text(pDX, IDC_NBCOLS, m_nbCols);
	DDX_Control(pDX, IDC_APPLY, m_btnApply);
	DDX_Control(pDX, IDC_ARROWUP, m_btnArrowUp);
	DDX_Control(pDX, IDC_ARROWDOWN, m_btnArrowDown);
	DDX_Control(pDX, IDC_HIDESHOWICONS, m_btnHideshowIcons);
	DDX_Control(pDX, IDC_SETCURRENT, m_btnSetCurrent);
	DDX_Text(pDX, IDC_XPOS, m_uXPos);
	DDX_Text(pDX, IDC_YPOS, m_uYPos);
	DDX_Slider(pDX, IDC_TRANSPARENCY, m_nTransparency);
	DDX_Check(pDX, IDC_BACKGROUNDTRANSPARENCY, m_nBackgroundTransparency);
	DDX_Check(pDX, IDC_ENABLECALENDAR, m_enableCalendar);
	DDX_Control(pDX, IDC_TREE1, m_Tree);
	DDX_Control(pDX, IDC_SPIN1, m_spin1);
	DDX_Control(pDX, IDC_SPIN2, m_spin2);
}

BEGIN_MESSAGE_MAP(CWallpaperQDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_LBN_SELCHANGE(IDC_BROWSELIST, OnLbnSelchangeBrowselist)	
	ON_BN_CLICKED(IDC_ADD, OnBnClickedAdd)
	ON_BN_CLICKED(IDC_QLISTEMPTY, OnBnClickedQlistempty)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_APPLY, OnBnClickedApply)
	ON_CBN_SELCHANGE(IDC_POSITION, OnCbnSelchangePosition)
	ON_COMMAND(ID_CONFIGUREWALLPAPERQUEUE, OnShowConfigWindow)
	ON_COMMAND(ID_SETWALLPAPERTONEXTIMAGE, OnSetToNextImage)
	ON_COMMAND(ID_OPTIONS, OnOptions)
	ON_COMMAND(ID_EXITWPQ, OnExit)
	ON_MESSAGE(WM_NI_MSG, OnNotifyIconMsg)
	ON_MESSAGE(WM_SHOWOPTIONS, OnShowOptions)
	ON_MESSAGE(WM_HIDEWINDOW, OnHideMainWindow)
	ON_LBN_SELCHANGE(IDC_QLIST, OnLbnSelchangeQlist)
	ON_BN_CLICKED(IDC_REMOVESELECTED, OnBnClickedRemoveselected)
	ON_BN_CLICKED(IDC_ARROWUP, OnBnClickedArrowup)
	ON_BN_CLICKED(IDC_ARROWDOWN, OnBnClickedArrowdown)
	ON_BN_CLICKED(IDC_RANDOMIZE, OnBnClickedRandomize)
	ON_BN_CLICKED(ID_CANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_HIDESHOWICONS, OnBnClickedHideshowicons)
	ON_BN_CLICKED(IDC_SETCURRENT, OnBnClickedSetCurrent)
	ON_BN_CLICKED(IDC_FONT_BTN, OnFontBtn)
	ON_BN_CLICKED(IDC_COLOR_BTN, OnColorBtn)
	ON_EN_CHANGE(IDC_XPOS, OnEnChangeXpos)
	ON_EN_CHANGE(IDC_YPOS, OnEnChangeYpos)
	ON_BN_CLICKED(IDC_BACKGROUNDTRANSPARENCY, OnBnClickedBackgroundtransparency)
	ON_NOTIFY(TVN_SELCHANGED, IDC_TREE1, OnSelchangedTree)
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_TREE1, OnItemexpandingTree)
	ON_BN_CLICKED(IDC_ENABLECALENDAR, OnBnClickedEnablecalendar)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN1, OnDeltaposSpin1)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN2, OnDeltaposSpin2)
	ON_EN_CHANGE(IDC_NBROWS, OnEnChangeNbrows)
	ON_EN_CHANGE(IDC_NBCOLS, OnEnChangeNbcols)
END_MESSAGE_MAP()

void CWallpaperQDlg::OnFontBtn()
{
	CFontDialog dlg(NULL);

	CClientDC dc(this);
	dlg.m_cf.lpLogFont->lfHeight = -MulDiv(m_nFontSize, dc.GetDeviceCaps(LOGPIXELSY), 72);
	dlg.m_cf.rgbColors = m_crTextColor;
	dlg.m_cf.Flags |= CF_INITTOLOGFONTSTRUCT;
	_tcscpy(dlg.m_cf.lpLogFont->lfFaceName, m_szFontName);
	if (dlg.DoModal() == IDOK)
	{
		m_szFontName = dlg.GetFaceName();
		m_nFontSize = dlg.GetSize() / 10;
		m_crTextColor = dlg.GetColor();
		DrawText();
		m_btnApply.EnableWindow();
	}
}

void CWallpaperQDlg::OnColorBtn()
{
	CColorDialog dlg;
	dlg.m_cc.Flags |= CC_RGBINIT;
	dlg.m_cc.rgbResult = m_crBackColor;
	if (dlg.DoModal() == IDOK)
	{
		m_crBackColor = dlg.GetColor();
		DrawText();
		m_btnApply.EnableWindow();
	}
}

// CWallpaperQDlg message handlers

BOOL CWallpaperQDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	hAOT = LoadLibrary(_T("AOTHook.dll"));

	if (hAOT != NULL)
	{
		LPSetHook pSetHook = (LPSetHook)GetProcAddress(hAOT, "SetHook");
		if (pSetHook != NULL)
		{
			(pSetHook)(this->m_hWnd);
		}
	}

	translate();

	m_spin1.SetRange(1, 20);
	m_spin2.SetRange(1, 20);

	m_buildDesktopImage->SetHWnd(this->m_hWnd);
	clock->Start();

	DWORD dwRandom = 0;
	TCHAR strBrowsePath[MAX_PATH + 1];
	BOOL bFirstRun = FALSE;

	// Initialize the strBrowsePath string to zeros.
	ZeroMemory(strBrowsePath, sizeof(char) * (MAX_PATH + 1));

	CString cstrCmdLine;

	// Get the command line.
	cstrCmdLine = AfxGetApp()->m_lpCmdLine;

	// Get the application path.
	GetModuleFileName(AfxGetInstanceHandle(), strBrowsePath, MAX_PATH + 1);
	cstrAppPath.Format(_T("%s"), strBrowsePath);
	cstrAppPath = cstrAppPath.Left(cstrAppPath.ReverseFind('\\') + 1);

	// Set bitmaps for the arrow buttons.
	m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
	m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));

	m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));

	// Initialize or reset dialog controls.
	ResetMainDialog();

	// Initially minimize the dialog if not the first run.
	bFirstRun = (CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("RunAtStartup"), 2) == 2);
	if (bFirstRun)
		PostMessage(WM_SHOWOPTIONS, 0, 0);
	else
	{
		// If we send a command line of "-noexit", then keep the window up.
		if (cstrCmdLine.CompareNoCase(_T("-noexit")) == 0)
			iExitAfter = 0;
		else
			PostMessage(WM_HIDEWINDOW);
	}

	// Change the wallpaper.
	if (iUseTimer == 1 && iMinutes > 0)
		uiTimer = (UINT)SetTimer(1, iMinutes * 60000, NULL);
	else
		ChangeWallpaper(m_chkRandomize.GetCheck());

	// get time
	time_t ltime;
	time(&ltime);
	// convert to local time
	struct tm* pTime;
	pTime = localtime(&ltime);

	int timerValue = pTime->tm_hour * 60 * 60 * 1000;
	timerValue += pTime->tm_min * 60 * 1000;
	timerValue += pTime->tm_sec * 1000;

	SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);


	//Update automatic
	iUpdate = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Update"), FALSE);
	//Display clock
	iDisplayClock = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayClock"), FALSE);
	iDisplayVisible = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayVisible"), FALSE);
	iDisplayMonitor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayMonitor"), FALSE);
	iDisplayTrayMe = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayTrayMe"), FALSE);

	CSliderCtrl* pSlider = (CSliderCtrl*)GetDlgItem(IDC_TRANSPARENCY);
	pSlider->SetRange(0, 255, true);
	pSlider->SetPos(m_nTransparency);

	// Display the tray icon.
	niData.cbSize = NOTIFYICONDATA_V1_SIZE;
	niData.hWnd = this->m_hWnd;
	niData.uID = IDR_MAINFRAME;
	niData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	niData.uCallbackMessage = WM_NI_MSG;
	niData.hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	_tcscpy(niData.szTip, _T("Wallpaper Q"));

	Shell_NotifyIcon(NIM_ADD, &niData);
	m_buildDesktopImage->SetIcon();

	if (iUpdate == TRUE)
	{
		CString buf;
		CAutoUpdater updater;
		CAutoUpdater::ErrorType err = updater.CheckForUpdate(_T("http://systrayplay.free.fr/"));
		switch (err)
		{
		case CAutoUpdater::Success:
			buf = m_Helper.MyLoadString(IDS_UPDATEOK);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			//			::PostMessage(pFrame->m_hWnd, WM_DESTROY, 0, 0);
			break;
		case CAutoUpdater::UpdateNotRequired:
			break;
		case CAutoUpdater::UpdateNotComplete:
			buf = m_Helper.MyLoadString(IDS_UPDATEKO);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		case CAutoUpdater::NoExecutableVersion:
			buf = m_Helper.MyLoadString(IDS_UPDATENOEXECUTABLEVERSION);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		default:
			buf = m_Helper.MyLoadString(IDS_UPDATECONNECTFAILED);
			AfxMessageBox(buf, MB_ICONINFORMATION | MB_OK);
			break;
		}
	}

	//
	//Setup the Image lists
	//
	m_ImageListTree.Create(IDB_FILE_TYPES, 17, 1, RGB(255, 255, 255));
	m_Tree.SetImageList(&m_ImageListTree, TVSIL_NORMAL);

	//
	//List the local drives
	//
	for (TCHAR cLetter = _T('A'); cLetter <= _T('Z'); cLetter++)
	{
		CString sDrive;
		sDrive.Format(_T("%c:"), cLetter);						
		UINT nType = GetDriveType(sDrive + _T("\\"));
		if (DRIVE_REMOVABLE <= nType && nType <= DRIVE_RAMDISK)
			InsertItem(TVI_ROOT, NULL, sDrive, nType);
	}

	//
	//Expand the Tree to the Inital Path.
	//
	int nLevel = 0;
	CString sDirName;
	HTREEITEM hCurrent = TVI_ROOT;					//Current item in the list being expanded.
	CString tmpstrBrowsePath = cstrBrowsePath;
	while (ParsePath(tmpstrBrowsePath, nLevel, &sDirName))
	{
		//Network or Drive expand
		if (!nLevel && sDirName.IsEmpty())
			sDirName = NETWORK_NEIGHBOUR;
		//Search for the matching tree item and expand
		HTREEITEM hItem = m_Tree.GetChildItem(hCurrent);	
		while (hItem)
		{
			if (sDirName.CompareNoCase(m_Tree.GetItemText(hItem)) == 0)
			{
				hCurrent = hItem;
				m_Tree.Expand(hCurrent, TVE_EXPAND);
				break;
			}
			hItem = m_Tree.GetNextSiblingItem(hItem);
		}
		nLevel++;
	}

	return TRUE;
}

void CWallpaperQDlg::translate()
{
	m_Helper.translateWindowText(this, IDD_WALLPAPERQ_DIALOG);
	m_Helper.translateDlgItemText(this, IDC_ADD);
	m_Helper.translateDlgItemText(this, IDC_LLOOK);
	m_Helper.translateDlgItemText(this, IDC_COLOR);
	m_Helper.translateDlgItemText(this, IDC_INFOIMG);
	m_Helper.translateDlgItemText(this, IDC_WIDTH);
	m_Helper.translateDlgItemText(this, IDC_HEIGHT);
	m_Helper.translateDlgItemText(this, IDC_INFOIMG2);
	m_Helper.translateDlgItemText(this, IDC_WIDTH2);
	m_Helper.translateDlgItemText(this, IDC_HEIGHT2);
	m_Helper.translateDlgItemText(this, IDC_IMGQUEUED);
	m_Helper.translateDlgItemText(this, IDC_TYPE);
	m_Helper.translateDlgItemText(this, IDC_TYPE2);
	m_Helper.translateDlgItemText(this, IDC_POS);
	m_Helper.translateDlgItemText(this, IDC_RANDOMIZE);
	m_Helper.translateDlgItemText(this, IDC_LNBROWS);
	m_Helper.translateDlgItemText(this, IDC_LNBCOLS);
	m_Helper.translateDlgItemText(this, IDC_SETCURRENT);
	m_Helper.translateDlgItemText(this, IDC_REMOVESELECTED);
	m_Helper.translateDlgItemText(this, IDC_QLISTEMPTY);
	m_Helper.translateDlgItemText(this, IDC_WALLPAPERTITTLE);
	m_Helper.translateDlgItemText(this, IDC_CALENDAR);
	m_Helper.translateDlgItemText(this, IDC_POSX);
	m_Helper.translateDlgItemText(this, IDC_POSY);
	m_Helper.translateDlgItemText(this, IDC_FONT);
	m_Helper.translateDlgItemText(this, IDC_FONT2);
	m_Helper.translateDlgItemText(this, IDC_FONT3);
	m_Helper.translateDlgItemText(this, IDC_BACKGROUNDTRANSPARENCY);
	m_Helper.translateDlgItemText(this, IDOK);
	m_Helper.translateDlgItemText(this, ID_CANCEL);
	m_Helper.translateDlgItemText(this, IDC_APPLY);

	m_comboPosition.ResetContent();
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_TILED));
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_CENTER));
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_STRETCH));
	m_comboPosition.AddString(m_Helper.MyLoadString(IDS_ADJUST));

	CString translation;
	CString name;
	name.Format(_T("T%d"), IDS_LAOT);
	translation = m_Helper.MyLoadString(IDS_LAOT);
	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), name, translation);
	name.Format(_T("T%d"), IDS_LTRAYME);
	translation = m_Helper.MyLoadString(IDS_LTRAYME);
	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), name, translation);
	name.Format(_T("T%d"), IDS_LSENDTOMONITOR);
	translation = m_Helper.MyLoadString(IDS_LSENDTOMONITOR);
	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), name, translation);
}

void CWallpaperQDlg::OnPaint()
{
	CDialog::OnPaint();

	// Make sure the preview gets redrawn.
	DrawPreview();
	DrawPreview2();
	DrawText();
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CWallpaperQDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CWallpaperQDlg::ResetMainDialog()
{
	// Get the browse path for the top list box.
	cstrBrowsePath = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("LastPath"), cstrAppPath);

	m_uXPos = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartX"), 100); 
	m_uYPos = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartY"), 100);
	m_szFontName = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Font Name"), _T("Monotype Corsiva"));
	m_nFontSize = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Size"), 24);
	m_crTextColor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Color"), (INT)RGB(0, 255, 0));
	m_crBackColor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Back Ground Color"), (INT)RGB(255, 255, 255));
	m_nTransparency = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Transparency"), 88);
	m_nBackgroundTransparency = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("BackgroundTransparency"), 0);
	m_enableCalendar = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("EnableCalendar"), 0);

	CDC* pDC = GetDC();  // Get the device context of this window.
	int iScreenWidth, iScreenHeight;
	CString cstrScreenRes, cstrColors;
	RECT rect;

	//// Get startup options.
	iStartup = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("RunAtStartup"), 1); 
	iThisUser = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ThisOrAllUsers"), 1);
	iExitAfter = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ExitAfter"), 0);
	iUseTimer = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("UseTimer"), 0);
	iMinutes = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("TimerMinutes"), 0);

	// Get the screen dimensions.
	iScreenWidth = GetDeviceCaps(pDC->m_hDC, HORZRES);
	iScreenHeight = GetDeviceCaps(pDC->m_hDC, VERTRES);

	// Display resolution and colors.
	cstrScreenRes.Format(_T("%d x %d"), iScreenWidth, iScreenHeight);
	m_stResolution.SetWindowText(cstrScreenRes);

	cstrColors.Format(_T("%d bit"), GetDeviceCaps(pDC->m_hDC, BITSPIXEL));
	m_stColors.SetWindowText(cstrColors);

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.

	// Move the window to the center of the screen.
	GetWindowRect(&rect);
	MoveWindow(iScreenWidth / 2 - (rect.right - rect.left) / 2, iScreenHeight / 2 - (rect.bottom - rect.top) / 2, (rect.right - rect.left), (rect.bottom - rect.top));

	// Set the scroll size of the browse list box.
	m_lbBrowseList.GetWindowRect(&rect);
	m_lbBrowseList.SetHorizontalExtent(rect.right - rect.left + 100);

	// Set the scroll size of the queue list box.
	m_lbQList.SetHorizontalExtent(m_lbBrowseList.GetHorizontalExtent());

	// Fill the top list box with all image files in the specified location.
	m_lbBrowseList.ResetContent();
	FillList();

	// Fill the queue list with currently queued images.
	FillQueuedImages();

	// Disable the position combobox since nothing is selected in the queue list. 
	m_comboPosition.EnableWindow(FALSE);

	// Set the randomize checkbox.
	m_chkRandomize.SetCheck((int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Randomize")));
	m_nbRows = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbRows"), 1);
	m_nbCols = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbCols"), 1);

	UpdateData(false);
	OnBnClickedEnablecalendar();

	m_btnApply.EnableWindow(FALSE);
}

void CWallpaperQDlg::FillQueuedImages()
{
	DWORD dwNumImagesInQueue = 0;
	CString cstrImageInfoPath = _T("");
	CString cstrImageInfo = _T("");
	int iQueuedWallpaperPosition = 0;

	// Get the total number of already queued images from the registry.
	dwNumImagesInQueue = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NumImages"));

	// Clear the queued image list.
	m_lbQList.ResetContent();

	// Fill the queued images list with already queued images from the registry.
	for (DWORD dwLoop = 0; dwLoop < dwNumImagesInQueue; dwLoop++)
	{
		cstrImageInfoPath.Format(_T("%d"), dwLoop);

		cstrImageInfo = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), cstrImageInfoPath);

		// We save the image path and also the "position" of the wallpaper in the same string (comma delimited).
		int iIndex = m_lbQList.InsertString((int)dwLoop, cstrImageInfo.Left(cstrImageInfo.Find(_T(","))));
		int iWPPos = _tstoi(cstrImageInfo.Right(cstrImageInfo.GetLength() - cstrImageInfo.Find(_T(",")) - 1));
		m_lbQList.SetItemData(iIndex, iWPPos);
	}

	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"));

	// Select the first item.
	if (dwNumImagesInQueue > 0)
	{
		for (int i = 0; i < m_lbQList.GetCount(); i++)
			m_lbQList.SetSel(i, i == iCurrentImage);
		m_lbQList.SetFocus();
		OnLbnSelchangeQlist();
	}
}

void CWallpaperQDlg::OnEnChangeLookin()
{
	// Change browse list contents based on what they typed in the edit box.
	m_lbBrowseList.ResetContent();
	OnLbnSelchangeBrowselist();
	FillList();
}

void CWallpaperQDlg::FillList()
{
	CFileFind finder;
	int iOnlyBMPs = 1;

	// Clear the list box.
	m_lbBrowseList.ResetContent();

	// Build the search string.
	if (cstrBrowsePath.GetAt(cstrBrowsePath.GetLength() - 1) != '\\')
		cstrBrowsePath += "\\";

	CString cstrSearch[6] =
	{
		cstrBrowsePath + _T("*.bmp"), cstrBrowsePath + _T("*.jpg"), cstrBrowsePath + _T("*.jpeg"), cstrBrowsePath + _T("*.gif"), cstrBrowsePath + _T("*.dib"), cstrBrowsePath + _T("*.png")
	};

	// If Active Desktop is available, we can use other images besides BMPs.
	if (pIActiveDesktop != NULL)
		iOnlyBMPs = 6;

	// Get all image files into the list.
	for (int i = 0; i < iOnlyBMPs; i++)
	{
		BOOL bWorking = finder.FindFile(cstrSearch[i]);
		while (bWorking)
		{
			bWorking = finder.FindNextFile();
			m_lbBrowseList.AddString(finder.GetFileName());
		}

		finder.Close();
	}

	// Select the first image.
	if (m_lbBrowseList.GetCount() > 0)
	{
		m_lbBrowseList.SetSel(0);
		m_lbBrowseList.SetFocus();
		OnLbnSelchangeBrowselist();
	}
}

void CWallpaperQDlg::OnLbnSelchangeBrowselist()
{
	// Change the previewed image.
	DrawPreview2();
}

void CWallpaperQDlg::DrawPreview()
{
	USES_CONVERSION;

	CString cstrWidth, cstrHeight, cstrType;
	UINT uiWidth, uiHeight;
	UINT thumbWidth, thumbHeight;
	int iAdjTop, iAdjLeft;
	int iCount = 0;
	double dRatio;
	CDC* pDC;
	TCHAR szBuf[MAX_PATH];
	RECT rect;
	CWnd* pWndTemp = NULL;
	BOOL bExists = TRUE;

	if (!IsWindowVisible())
		return;

	// Initialize the Graphics class in GDI+.
	pDC = m_picPreview.GetWindowDC();
	Graphics graphics(pDC->m_hDC);

	// Fill the preview ara with a WHITE background.
	m_picPreview.GetClientRect(&rect);
	rect.left += 1;
	rect.top += 1;
	rect.right -= 1;
	rect.bottom -= 1;
	pDC->FillSolidRect(&rect, RGB(255, 255, 255));

	iCount = m_lbQList.GetCount();  // QList has focus, so get its count.

	if (iCount > 0)
	{
		// Get the path to the image from the list with focus.
		m_lbQList.GetText(m_lbQList.GetCurSel(), szBuf);

		// Make sure there is an image chosen.
		if (szBuf[0] != '/0')
		{
			// Make sure the image exists
			CFile file;
			if (file.Open(szBuf, CFile::modeRead))
				file.Close();
			else
				bExists = FALSE;

			if (bExists)
			{
				try
				{
					// Use the Image class to display a thumbnail of the image.
					Image image(T2CW(szBuf));

					// Determine the appropriate size of the thumbnail preview
					// given the image size ratio and the preview window size ratio.
					uiWidth = image.GetWidth();
					uiHeight = image.GetHeight();
					dRatio = ((double)uiWidth) / ((double)uiHeight);

					// If the width is larger than the height of the image, 
					// set the width of the thumbnail to the width of the preview area
					// and calculate the thumbnail height by using the ratios.
					// If the height is larger than the width of the image, 
					// set the height of the thumbnail to the height of the preview area
					// and calculate the thumbnail width by using the ratios.
					if (uiWidth > uiHeight)
					{
						thumbWidth = rect.right - rect.left;
						thumbHeight = (UINT)(thumbWidth / dRatio);

						if (thumbHeight == 0)
							thumbHeight = 1;
						if (thumbHeight > (UINT)(rect.bottom - rect.top))
							thumbHeight = rect.bottom - rect.top;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1 + ((rect.bottom - rect.top) / 2) - (thumbHeight / 2);
						iAdjLeft = 1;
					}
					else
					{
						thumbHeight = rect.bottom - rect.top;
						thumbWidth = (UINT)(uiWidth * thumbHeight / uiHeight);

						if (thumbWidth == 0)
							thumbWidth = 1;
						if (thumbWidth > (UINT)(rect.right - rect.left))
							thumbWidth = rect.right - rect.left;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1;
						iAdjLeft = 1 + ((rect.right - rect.left) / 2) - (thumbWidth / 2);
					}

					// Get the thumbnail and display it in the preview control.
					Image* pThumbnail = image.GetThumbnailImage(thumbWidth, thumbHeight, NULL, NULL);
					graphics.DrawImage(pThumbnail, iAdjLeft, iAdjTop, pThumbnail->GetWidth(), pThumbnail->GetHeight());

					// Display the image width and height.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH);
					cstrWidth.Format(_T("%s %d"), buf, uiWidth);
					m_stWidth.SetWindowText(cstrWidth);
					buf = m_Helper.MyLoadString(IDC_HEIGHT);
					cstrHeight.Format(_T("%s %d"), buf, uiHeight);
					m_stHeight.SetWindowText(cstrHeight);

					// Display the image bits per pixel (color depth).
					buf = m_Helper.MyLoadString(IDC_TYPE);
					switch (image.GetPixelFormat())
					{
					case PixelFormat1bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_1BPP));
						break;
					case PixelFormat4bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_4BPP));
						break;
					case PixelFormat8bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_8BPP));
						break;
					case PixelFormat16bppARGB1555:
					case PixelFormat16bppGrayScale:
					case PixelFormat16bppRGB555:
					case PixelFormat16bppRGB565:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_16BPP));
						break;
					case PixelFormat24bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_24BPP));
						break;
					case PixelFormat32bppARGB:
					case PixelFormat32bppPARGB:
					case PixelFormat32bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_32BPP));
						break;
					case PixelFormat48bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_48BPP));
						break;
					case PixelFormat64bppARGB:
					case PixelFormat64bppPARGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_64BPP));
						break;
					default:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_UNKNOWN));
						break;
					}

					m_stType.SetWindowText(cstrType);
				}
				catch (...)
				{
				}
			}
			else
			{
				COLORREF oldCR = pDC->SetTextColor(RGB(255, 0, 0));
				//TODO
				pDC->DrawText(m_Helper.MyLoadString(IDS_FILENOTEXIST), &rect, DT_CENTER | DT_WORDBREAK);
				pDC->SetTextColor(oldCR);

				// No image to preview.
				CString buf = m_Helper.MyLoadString(IDC_WIDTH);
				m_stWidth.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_HEIGHT);
				m_stHeight.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_TYPE);
				m_stType.SetWindowText(buf);
			}
		}
		else
		{
			// No image to preview.
			CString buf = m_Helper.MyLoadString(IDC_WIDTH);
			m_stWidth.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_HEIGHT);
			m_stHeight.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_TYPE);
			m_stType.SetWindowText(buf);
		}
	}

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.
}

void CWallpaperQDlg::DrawPreview2()
{
	USES_CONVERSION;

	CString cstrWidth, cstrHeight, cstrType;
	UINT uiWidth, uiHeight;
	UINT thumbWidth, thumbHeight;
	int iAdjTop, iAdjLeft;
	int iCount = 0;
	double dRatio;
	CDC* pDC;
	TCHAR szBuf[MAX_PATH];
	RECT rect;
	CWnd* pWndTemp = NULL;
	BOOL bExists = TRUE;

	if (!IsWindowVisible())
		return;

	// Initialize the Graphics class in GDI+.
	pDC = m_picPreview2.GetWindowDC();
	Graphics graphics(pDC->m_hDC);

	// Fill the preview ara with a WHITE background.
	m_picPreview.GetClientRect(&rect);
	rect.left += 1;
	rect.top += 1;
	rect.right -= 1;
	rect.bottom -= 1;
	pDC->FillSolidRect(&rect, RGB(255, 255, 255));

	iCount = m_lbBrowseList.GetCount();  // LeftList has focus, so get its count.

	if (iCount > 0)
	{
		// Get the path to the image from the list with focus.
		_tcscpy(szBuf, cstrBrowsePath.GetString());
		TCHAR* tmp = szBuf + cstrBrowsePath.GetLength();
		m_lbBrowseList.GetText(m_lbBrowseList.GetCurSel(), tmp);

		// Make sure there is an image chosen.
		if (szBuf[0] != '/0')
		{
			// Make sure the image exists
			CFile file;
			if (file.Open(szBuf, CFile::modeRead))
				file.Close();
			else
				bExists = FALSE;

			if (bExists)
			{
				try
				{
					// Use the Image class to display a thumbnail of the image.
					Image image(T2CW(szBuf));

					// Determine the appropriate size of the thumbnail preview
					// given the image size ratio and the preview window size ratio.
					uiWidth = image.GetWidth();
					uiHeight = image.GetHeight();
					dRatio = ((double)uiWidth) / ((double)uiHeight);

					// If the width is larger than the height of the image, 
					// set the width of the thumbnail to the width of the preview area
					// and calculate the thumbnail height by using the ratios.
					// If the height is larger than the width of the image, 
					// set the height of the thumbnail to the height of the preview area
					// and calculate the thumbnail width by using the ratios.
					if (uiWidth > uiHeight)
					{
						thumbWidth = rect.right - rect.left;
						thumbHeight = (UINT)(thumbWidth / dRatio);

						if (thumbHeight == 0)
							thumbHeight = 1;
						if (thumbHeight > (UINT)(rect.bottom - rect.top))
							thumbHeight = rect.bottom - rect.top;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1 + ((rect.bottom - rect.top) / 2) - (thumbHeight / 2);
						iAdjLeft = 1;
					}
					else
					{
						thumbHeight = rect.bottom - rect.top;
						thumbWidth = (UINT)(uiWidth * thumbHeight / uiHeight);

						if (thumbWidth == 0)
							thumbWidth = 1;
						if (thumbWidth > (UINT)(rect.right - rect.left))
							thumbWidth = rect.right - rect.left;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1;
						iAdjLeft = 1 + ((rect.right - rect.left) / 2) - (thumbWidth / 2);
					}

					// Get the thumbnail and display it in the preview control.
					Image* pThumbnail = image.GetThumbnailImage(thumbWidth, thumbHeight, NULL, NULL);
					graphics.DrawImage(pThumbnail, iAdjLeft, iAdjTop, pThumbnail->GetWidth(), pThumbnail->GetHeight());

					// Display the image width and height.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
					cstrWidth.Format(_T("%s %d"), buf, uiWidth);
					m_stWidth2.SetWindowText(cstrWidth);
					buf = m_Helper.MyLoadString(IDC_HEIGHT2);
					cstrHeight.Format(_T("%s %d"), buf, uiHeight);
					m_stHeight2.SetWindowText(cstrHeight);

					// Display the image bits per pixel (color depth).
					buf = m_Helper.MyLoadString(IDC_TYPE2);
					switch (image.GetPixelFormat())
					{
					case PixelFormat1bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_1BPP));
						break;
					case PixelFormat4bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_4BPP));
						break;
					case PixelFormat8bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_8BPP));
						break;
					case PixelFormat16bppARGB1555:
					case PixelFormat16bppGrayScale:
					case PixelFormat16bppRGB555:
					case PixelFormat16bppRGB565:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_16BPP));
						break;
					case PixelFormat24bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_24BPP));
						break;
					case PixelFormat32bppARGB:
					case PixelFormat32bppPARGB:
					case PixelFormat32bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_32BPP));
						break;
					case PixelFormat48bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_48BPP));
						break;
					case PixelFormat64bppARGB:
					case PixelFormat64bppPARGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_64BPP));
						break;
					default:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_UNKNOWN));
						break;
					}

					m_stType2.SetWindowText(cstrType);
				}
				catch (...)
				{
				}
			}
			else
			{
				COLORREF oldCR = pDC->SetTextColor(RGB(255, 0, 0));
				//TODO
				pDC->DrawText(m_Helper.MyLoadString(IDS_FILENOTEXIST), &rect, DT_CENTER | DT_WORDBREAK);
				pDC->SetTextColor(oldCR);

				// No image to preview.
				CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
				m_stWidth2.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_HEIGHT2);
				m_stHeight2.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_TYPE2);
				m_stType2.SetWindowText(buf);
			}
		}
		else
		{
			// No image to preview.
			CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
			m_stWidth2.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_HEIGHT2);
			m_stHeight2.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_TYPE2);
			m_stType2.SetWindowText(buf);
		}
	}

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.
}

void CWallpaperQDlg::OnBnClickedAdd()
{
	int iCount;
	CString cstrTemp;

	// Get all selected items from the browse list.
	iCount = m_lbBrowseList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbBrowseList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and add it to the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
	{
		m_lbBrowseList.GetText(aryListBoxSel.GetAt(i), cstrTemp);

		// Select each newly added item.
		if (m_lbQList.FindStringExact(0, cstrBrowsePath + cstrTemp) > -1)
		{
			// If its already in the list, ask if they want to add it again.
			if (AfxMessageBox(m_Helper.MyLoadString(IDS_ALREADYEXIST), MB_YESNO) == IDYES)
			{
				int index = m_lbQList.InsertString(m_lbQList.GetCount() + 1, cstrBrowsePath + cstrTemp);
				m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrBrowsePath + cstrTemp));
			}
		}
		else
		{
			//int index = m_lbQList.InsertString(m_lbQList.GetCount() + 1, cstrTemp);
			int index = m_lbQList.AddString(cstrBrowsePath + cstrTemp);
			m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrBrowsePath + cstrTemp));
		}
	}

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnLbnSelchangeQlist()
{
	// Show the last selected item in the queued image list in the preview window.
	DrawPreview();

	if (m_lbQList.GetCount() > 0)
	{
		// If there are items in the queue list, enable the position combo box. 
		m_comboPosition.SetCurSel((int)m_lbQList.GetItemData(m_lbQList.GetCurSel()));
		m_comboPosition.EnableWindow();

		// Change the up and down arrow images.
		if (m_lbQList.GetSelCount() > 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));
		}
		else if (m_lbQList.GetSelCount() == 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWDOWN)));
		}
	}
}

void CWallpaperQDlg::OnBnClickedArrowup()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) up one spot in the list.
	if (m_lbQList.GetCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex > 0)
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex--;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			m_btnApply.EnableWindow();
		}
	}
}

void CWallpaperQDlg::OnBnClickedArrowdown()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) down one spot in the list.
	if (m_lbQList.GetSelCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex < m_lbQList.GetCount())
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex++;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			m_btnApply.EnableWindow();
		}
	}
}


void CWallpaperQDlg::OnBnClickedRemoveselected()
{
	int iCount = 0;

	// Get all selected items from the browse list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and delete it from the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.DeleteString(aryListBoxSel.GetAt(i));

	m_btnApply.EnableWindow();
}


void CWallpaperQDlg::OnBnClickedQlistempty()
{
	// Empty the Queued images list.
	m_lbQList.ResetContent();

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnCbnSelchangePosition()
{
	int iPosition = 1;
	int iCount = 0;

	iPosition = m_comboPosition.GetCurSel();

	// Get all selected items from the queued image list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and change its wallpaper position.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.SetItemData(aryListBoxSel.GetAt(i), (DWORD)iPosition);

	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnBnClickedRandomize()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnSetToNextImage()
{
	ApplyChanges();
}

void CWallpaperQDlg::ApplyChanges(BOOL bChangeWallpaper /*= TRUE*/)
{
	// Delete all keys under wallpaperq in the registry.
	CRegistry::RegDelete(_T("Software\\Desktop Calendar_V1.0"), _T(""), _T("WallpaperQ"));

	// Set the last browse path in the registry.
	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("LastPath"), cstrBrowsePath);

	// Set the number of images saved in the registry from the queue list.
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NumImages"), m_lbQList.GetCount());

	// Loop through the queued image list and save each image in the registry.
	for (int iLoop = 0; iLoop < m_lbQList.GetCount(); iLoop++)
	{
		CString cstrPath;
		CString cstrRegValue = _T("");
		CString cstrRegPath = _T("");

		m_lbQList.GetText(iLoop, cstrPath);
		cstrRegPath.Format(_T("%s,%d"), cstrPath, (int)m_lbQList.GetItemData(iLoop));

		cstrRegValue.Format(_T("%d"), iLoop);

		CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), cstrRegValue, cstrRegPath);
	}

	// Set whether to display a random image when run.
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Randomize"), (DWORD)m_chkRandomize.GetCheck());
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbRows"), (DWORD)m_nbRows);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbCols"), (DWORD)m_nbCols);

	// Set them in the registry.
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("RunAtStartup"), (DWORD)iStartup); 
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ThisOrAllUsers"), (DWORD)iThisUser);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ExitAfter"), (DWORD)iExitAfter);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("UseTimer"), (DWORD)iUseTimer);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("TimerMinutes"), (DWORD)iMinutes);

	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Font Name"), m_szFontName);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Size"), m_nFontSize);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Text Color"), m_crTextColor);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Back Ground Color"), m_crBackColor);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Transparency"), m_nTransparency);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("BackgroundTransparency"), m_nBackgroundTransparency);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("EnableCalendar"), m_enableCalendar);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartX"), m_uXPos);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("StartY"), m_uYPos);


	// Change the wallpaper.
	if (bChangeWallpaper)
		ChangeWallpaper(m_chkRandomize.GetCheck());
}

void CWallpaperQDlg::OnBnClickedApply()
{
	// Apply changes without closing/minimizing.
	UpdateData(true);
	ApplyChanges();

	m_btnApply.EnableWindow(FALSE);
}

void CWallpaperQDlg::ChangeWallpaper(int iMode)
{
	USES_CONVERSION;

	int iImageNum = 0;
	int iPosition = 1;
	CString cstrTileWallpaper;
	CString cstrWallpaperStyle;
	TCHAR szBuf[MAX_PATH];
	m_buildDesktopImage->ReadRegistry();
	m_buildDesktopImage->StartBitmap();

	// Choose then next image in the list, selected image, or choose a random image.
	if (iMode == 1)  // random
	{
		srand((unsigned)time(NULL));

		iImageNum = rand() % m_lbQList.GetCount();

		iCurrentImage = iImageNum;
	}
	else if (iMode == 2)
	{
		iCurrentImage = m_lbQList.GetCurSel();

		iImageNum = iCurrentImage;
	}
	else
	{
		iCurrentImage++;
		if (iCurrentImage + 1 > m_lbQList.GetCount())
			iCurrentImage = 0;

		iImageNum = iCurrentImage;
	}
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"), (DWORD)iCurrentImage);

	int xCount = m_nbRows;
	int yCount = m_nbCols;
	for (int i = 0; i< m_buildDesktopImage->m_MonitorArray.GetSize(); i++)
	{
		for (int k = 0; k < xCount; k++)
		{
			for (int l = 0; l < yCount; l++)
			{
				if (i > 0 || k > 0 || l > 0)
				{
					// Choose then next image in the list, selected image, or choose a random image.
					if (iMode == 1)  // random
					{
						iImageNum = rand() % m_lbQList.GetCount();
					}
					else
					{
						iImageNum++;
						if (iImageNum + 1 > m_lbQList.GetCount())
							iImageNum = 0;
					}
				}
				// Get the image path from the registry by the image number.
				iPosition = (int)m_lbQList.GetItemData(iImageNum);
				m_lbQList.GetText(iImageNum, szBuf);
				CString tmp;
				tmp.Format(_T("%s"), szBuf);
				m_buildDesktopImage->SetImage(tmp, iPosition, i, k, l, xCount, yCount);
			}
		}
	}

	m_buildDesktopImage->StopBitmap();

	TCHAR szSysDir[MAX_PATH];
	GetSystemDirectory(szSysDir, MAX_PATH);
	CString szNewFile = szSysDir;
	szNewFile += _T("\\Desktop Calendar.bmp");
	_tcscpy(szBuf, szNewFile.operator LPCTSTR());

	if (szBuf[0] != '/0')
	{
		// If Active Desktop is not available, use the SystemParametersInfo function.
		if (pIActiveDesktop == NULL)
		{
			if (iPosition == 0)
			{
				// Tiled
				cstrTileWallpaper = _T("1");
				cstrWallpaperStyle = _T("1");
			}
			else if (iPosition == 1)
			{
				// Center
				cstrTileWallpaper = _T("0");
				cstrWallpaperStyle = _T("1");
			}
			else
			{
				// Stretch
				cstrTileWallpaper = _T("0");
				cstrWallpaperStyle = _T("2");
			}
			cstrTileWallpaper = _T("1");
			cstrWallpaperStyle = _T("1");

			// Set registry values for the wallpaper position.
			CRegistry::SetRegString(_T("Control Panel\\Desktop"), _T("TileWallpaper"), cstrTileWallpaper);
			CRegistry::SetRegString(_T("Control Panel\\Desktop"), _T("WallpaperStyle"), cstrWallpaperStyle);

			// Change the wallpaper.
			SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (void*)szBuf, SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
		}
		else
		{
			WALLPAPEROPT wpOptions;
			COMPONENTSOPT compOptions;

			compOptions.dwSize = sizeof(COMPONENTSOPT);
			compOptions.fActiveDesktop = TRUE;
			compOptions.fEnableComponents = TRUE;

			pIActiveDesktop->SetDesktopItemOptions(&compOptions, 0);

			// Set the wallpaper position.
			wpOptions.dwSize = sizeof(WALLPAPEROPT);
			if (iPosition == 0)
				wpOptions.dwStyle = WPSTYLE_TILE;
			else if (iPosition == 1)
				wpOptions.dwStyle = WPSTYLE_CENTER;
			else
				wpOptions.dwStyle = WPSTYLE_STRETCH;
			wpOptions.dwStyle = WPSTYLE_TILE;

			pIActiveDesktop->SetWallpaperOptions(&wpOptions, 0);

			// Set the wallpaper image.
			pIActiveDesktop->SetWallpaper(T2CW(szBuf), 0);

			// Apply all changes.
			pIActiveDesktop->ApplyChanges(AD_APPLY_ALL);
		}

		if (iExitAfter == 1)
			OnExit();
	}
}

LRESULT CWallpaperQDlg::OnNotifyIconMsg(WPARAM wParam, LONG lParam)
{
	// Show a right-click menu from the system tray icon 
	// if they right-click or double click it.
	switch (lParam)
	{
	case WM_LBUTTONDBLCLK:
		SetForegroundWindow();
		OnShowConfigWindow();
		break;
	case WM_RBUTTONUP:
		POINT point;
		CMenu menu;

		GetCursorPos(&point);

		menu.LoadMenu(IDR_NOTIFYICON_MENU);
		m_Helper.translateMenu(&menu, IDR_NOTIFYICON_MENU);
		CMenu* pMenu = menu.GetSubMenu(0);
		pMenu->SetDefaultItem(ID_CONFIGUREWALLPAPERQUEUE);

		SetForegroundWindow();

		pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);

		PostMessage(WM_NULL, 0, 0);
		menu.DestroyMenu();
		break;
	}

	return 1;
}

void CWallpaperQDlg::OnShowConfigWindow()
{
	RECT trayrect;

	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"));

	// Select it in the list
	for (int i = 0; i < m_lbQList.GetCount(); i++)
		m_lbQList.SetSel(i, i == iCurrentImage);
	m_lbQList.SetFocus();
	OnLbnSelchangeQlist();

	GetTrayWndRect(&trayrect);

	DrawAnimatedRects(IDANI_CAPTION, &trayrect, &rcLastSize);

	MoveWindow(&rcLastSize);
}

LRESULT CWallpaperQDlg::OnHideMainWindow(WPARAM wParam, LONG lParam)
{
	RECT rcTray, rcWindow;

	GetTrayWndRect(&rcTray);
	GetWindowRect(&rcWindow);

	rcLastSize = rcWindow;

	MoveWindow(rcTray.left, rcTray.top, 0, 0);

	DrawAnimatedRects(IDANI_CAPTION, &rcWindow, &rcTray);

	return 1;
}

LRESULT CWallpaperQDlg::OnShowOptions(WPARAM wParam, LONG lParam)
{
	// Display the options dialog.
	OnOptions();

	return 1;
}

void CWallpaperQDlg::OnOptions()
{
	CString cstrStartupApp = _T("");

	// Get initial options values from the registry.
	iStartup = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("RunAtStartup"), 1); 
	iThisUser = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ThisOrAllUsers"), 1);
	iExitAfter = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ExitAfter"), 0);
	iUseTimer = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("UseTimer"), 0);
	iMinutes = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("TimerMinutes"), 0);

	// Open the options dialog, sending the options values.
	COptionsDialog dlgOptions(iStartup, iThisUser, iExitAfter, iUseTimer, iMinutes, iUpdate, iDisplayClock, iDisplayVisible, iDisplayMonitor, iDisplayTrayMe);

	dlgOptions.Language = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Language"), _T(""));

	if (dlgOptions.DoModal() == IDOK)
	{
		// Get the chosen options.
		iStartup = dlgOptions.GetRunAtStartup(); 
		iThisUser = dlgOptions.GetRunForThisUserOrAll();
		iExitAfter = dlgOptions.GetExitAfterWallpaperChange();
		iUseTimer = dlgOptions.GetUseTimer();
		iMinutes = dlgOptions.GetMinutesForTimer();
		iUpdate = dlgOptions.GetUpdate();
		iDisplayClock = dlgOptions.GetDisplayClock();
		iDisplayVisible = dlgOptions.GetDisplayVisible();
		iDisplayMonitor = dlgOptions.GetDisplayMonitor();
		iDisplayTrayMe = dlgOptions.GetDisplayTrayMe();
		clock->ShowWindowClock(iDisplayClock);
		CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Language"), dlgOptions.Language); 
		translate();

		// Set them in the registry.
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("RunAtStartup"), (DWORD)iStartup); 
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ThisOrAllUsers"), (DWORD)iThisUser);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("ExitAfter"), (DWORD)iExitAfter);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("UseTimer"), (DWORD)iUseTimer);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("TimerMinutes"), (DWORD)iMinutes);

		//TODO
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Update"), (DWORD)iUpdate);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayClock"), (DWORD)iDisplayClock);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayVisible"), (DWORD)iDisplayVisible);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayMonitor"), (DWORD)iDisplayMonitor);
		CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("DisplayTrayMe"), (DWORD)iDisplayTrayMe);

		// Remove wallpaperq from the startup portion of the registry.
		CRegistry::RegDelete(_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"), _T("WallpaperQ"), _T(""), HKEY_LOCAL_MACHINE); 
		CRegistry::RegDelete(_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"), _T("WallpaperQ"), _T(""), HKEY_CURRENT_USER);

		// If the chose to start the app on system startup, add it back to the registry.
		if (iStartup == 1)
		{
			TCHAR path[MAX_PATH];
			::GetModuleFileName(AfxGetApp()->m_hInstance, path, MAX_PATH);
			cstrStartupApp.Format(_T("\"%s\""), path);
			if (iThisUser == 1)
			{
				// For this user only.
				CRegistry::SetRegString(_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"), _T("WallpaperQ"), cstrStartupApp);
			}
			else
			{
				// For all users.
				CRegistry::SetRegString(_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"), _T("WallpaperQ"), cstrStartupApp, HKEY_LOCAL_MACHINE);
			}
		}

		// If they chose to change the wallpaper every X minutes, kill any 
		// previous timers and start a new one based on the minutes entered.
		if (iUseTimer == 1)
		{
			if (uiTimer != 0)
				KillTimer(uiTimer);

			if (iMinutes > 0)
				uiTimer = (UINT)SetTimer(1, iMinutes * 60000, NULL);
		}
		else
		{
			// Or just kill it if they didn't choose to use a timer.
			if (uiTimer != 0)
				KillTimer(uiTimer);
		}

		// Ask them if they want to exit now if they chose to exit after changing wallpaper.
		if (iExitAfter == 1)
		{
			if (AfxMessageBox(m_Helper.MyLoadString(IDS_EXITAFTERCHANGE), MB_YESNO) == IDYES)
				OnExit();
		}
	}
}

void CWallpaperQDlg::OnTimer(UINT nIDEvent)
{
	CDialog::OnTimer(nIDEvent);

	if (nIDEvent == 1)
	{
		// When the timer is fired, change the wallpaper to the next image.
		ChangeWallpaper(m_chkRandomize.GetCheck());
	}
	else if (nIDEvent == 2)
	{
		// show desktop icons after 2 minutes elapsed
		CWnd* pProgMan = FindWindowEx(NULL, NULL, _T("ProgMan"), _T("Program Manager"));
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, _T("SHELLDLL_DefView"), _T(""));
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, _T("SysListView32"), _T("FolderView"));
		pFolderView->ShowWindow(SW_SHOW);

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));

		KillTimer(2);
	}
	else if (nIDEvent == 3)
	{
		// When the timer is fired, change the wallpaper to the next image.
		ChangeWallpaper(m_chkRandomize.GetCheck());
		KillTimer(3);

		// get time
		time_t ltime;
		time(&ltime);
		// convert to local time
		struct tm* pTime;
		pTime = localtime(&ltime);

		int timerValue = pTime->tm_hour * 60 * 60 * 1000;
		timerValue += pTime->tm_min * 60 * 1000;
		timerValue += pTime->tm_sec * 1000;

		SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);
	}
}

void CWallpaperQDlg::OnBnClickedOk()
{
	// Apply changes, then minimize.
	ApplyChanges(m_btnApply.IsWindowEnabled());

	m_btnApply.EnableWindow(FALSE);

	SendMessage(WM_HIDEWINDOW);
}

void CWallpaperQDlg::OnExit()
{
	SendMessage(WM_CLOSE);
}

void CWallpaperQDlg::OnBnClickedCancel()
{
	SendMessage(WM_HIDEWINDOW);

	// They hit cancel, so we reset things.
	// If they clicked OK, it comes to this function, but it should have 
	// saved everything prior toi this.
	ResetMainDialog();
}

int CWallpaperQDlg::GetWPPositionByImageSize(CString cstrImagePath)
{
	USES_CONVERSION;

	CDC* pDC = NULL;
	UINT uiWidth;
	UINT uiHeight;
	int iVertRes;
	int iHorzRes;
	int iTemp;
	double dRatio;
	double dScreenRatio;
	int iCalcPosition = 2;

	// Initialize the Graphics class in GDI+.
	pDC = GetDC();
	Graphics graphics(pDC->m_hDC);

	// Use the Image class to get dimensions of the image.
	Image image(T2CW((LPCTSTR)cstrImagePath));

	uiWidth = image.GetWidth();
	uiHeight = image.GetHeight();
	dRatio = ((double)uiWidth / (double)uiHeight);
	// round it
	iTemp = (int)((dRatio * 100.0) + 0.51);
	dRatio = ((double)iTemp) / 100.0;

	iHorzRes = pDC->GetDeviceCaps(HORZRES);
	iVertRes = pDC->GetDeviceCaps(VERTRES);
	dScreenRatio = ((double)iHorzRes / (double)iVertRes);
	// round it
	iTemp = (int)((dScreenRatio * 100.0) + 0.51);
	dScreenRatio = ((double)iTemp) / 100.0;

	if (dRatio == dScreenRatio || dRatio > 1.0)
		iCalcPosition = 2; // stretch
	else
		iCalcPosition = 1; // center

	ReleaseDC(pDC);

	return iCalcPosition;
}

void CWallpaperQDlg::OnBnClickedHideshowicons()
{
	if (bIconsShown)
	{
		// minimize windows
		MinimizeAllWindows();
		OnBnClickedCancel();

		// hide desktop icons
		CWnd* pProgMan = FindWindowEx(NULL, NULL, _T("ProgMan"), _T("Program Manager"));
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, _T("SHELLDLL_DefView"), _T(""));
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, _T("SysListView32"), _T("FolderView"));
		pFolderView->ShowWindow(SW_HIDE);

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_SHOWICONS)));

		// set timer to reset the icons after 2 minutes
		SetTimer(2, 120000, NULL);
	}
	else
	{
		// show desktop icons
		CWnd* pProgMan = FindWindowEx(NULL, NULL, _T("ProgMan"), _T("Program Manager"));
		CWnd* pSHELLDLL_DefView = FindWindowEx(pProgMan->GetSafeHwnd(), NULL, _T("SHELLDLL_DefView"), _T(""));
		CWnd* pFolderView = FindWindowEx(pSHELLDLL_DefView->GetSafeHwnd(), NULL, _T("SysListView32"), _T("FolderView"));
		pFolderView->ShowWindow(SW_SHOW);

		// restore all windows
		MinimizeAllWindows(FALSE);

		this->BringWindowToTop();

		// change bitmap on button
		m_btnHideshowIcons.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_HIDEICONS)));
	}

	bIconsShown = !bIconsShown;
}

void CWallpaperQDlg::MinimizeAllWindows(BOOL bMinimize /*= TRUE*/)
{
	// minimize or restore opened windows
	CLSID clsid;
	IUnknown* pUnk = NULL;

	// get class id of Shell.Application to call method
	CLSIDFromProgID(L"Shell.Application", &clsid);

	// create an object of Shell.Application interface
	HRESULT hr = ::CoCreateInstance(clsid, NULL, CLSCTX_SERVER, IID_IUnknown, (void**)&pUnk);

	if (SUCCEEDED(hr))
	{
		// find and invoke the proper method
		IDispatch* pDispApp;
		hr = pUnk->QueryInterface(IID_IDispatch, (void**)&pDispApp);

		if (SUCCEEDED(hr))
		{
			DISPPARAMS dpNoArgs =
			{
				NULL, NULL, 0, 0
			};
			VARIANT vResult;

			IDispatch* pDispMinimizeAll = NULL;
			DISPID dispidMinimizeAll;

			OLECHAR* szFunction = NULL;

			if (bMinimize)
				szFunction = OLESTR("MinimizeAll");
			else
				szFunction = OLESTR("UndoMinimizeALL");

			hr = pDispApp->GetIDsOfNames(IID_NULL, &szFunction, 1, LOCALE_USER_DEFAULT, &dispidMinimizeAll);
			if (SUCCEEDED(hr))
			{
				hr = pDispApp->Invoke(dispidMinimizeAll, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &dpNoArgs, &vResult, NULL, NULL);
			}

			pDispApp->Release();
		}

		pUnk->Release();
	}
}

void CWallpaperQDlg::OnBnClickedSetCurrent()
{
	// tell the ChangeWallpaper function to set the wallpaper to the current selection inthe Q list
	UpdateData(true);
	ApplyChanges(FALSE);
	ChangeWallpaper(2);

	m_lbQList.SetFocus();
	Invalidate(FALSE);
}

void CWallpaperQDlg::DrawText()
{
	CWnd* pWnd;
	pWnd = GetDlgItem(IDC_SAMPLE_TEXT);
	if (pWnd)
	{
		CDC* pDC = pWnd->GetDC();
		CFont Font;
		Font.CreateFont(m_nFontSize, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, FF_DONTCARE, m_szFontName); 
		pDC->SelectObject(&Font);
		TEXTMETRIC tmText;
		pDC->GetTextMetrics(&tmText);
		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(m_crTextColor);

		INT m_nCharWidth = (tmText.tmMaxCharWidth + 2 * tmText.tmAveCharWidth);	
		INT m_nCharHeight = tmText.tmHeight + 4;

		CRect TextRect;
		CRect ClientRect;
		pWnd->GetClientRect(&ClientRect);
		int nStartX = (ClientRect.Width() - m_nCharWidth * 6) / 2;
		if (nStartX < 0)
			nStartX = 2;
		int nStartY = (ClientRect.Height() - m_nCharHeight) / 2;
		if (nStartY < 0)
			nStartY = 2;

		TextRect.left = nStartX; 
		TextRect.top = nStartY ;
		TextRect.right = TextRect.left + m_nCharWidth * 6;
		TextRect.bottom = TextRect.top + m_nCharHeight;
		CBrush Brush;
		Brush.CreateSolidBrush(m_crBackColor);
		pDC->FillRect(&ClientRect, &Brush);
		//TODO
		pDC->DrawText(m_Helper.MyLoadString(IDS_EXEMPLE), &TextRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
	}
}
void CWallpaperQDlg::OnEnChangeXpos()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnEnChangeYpos()
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnBnClickedBackgroundtransparency()
{
	m_btnApply.EnableWindow();
}

HTREEITEM CWallpaperQDlg::InsertItem(HTREEITEM hParent, NETRESOURCE *const pNetResource, CString sText, int iImage, int iImageSelected /* = -1 */)
{
	TVINSERTSTRUCT InsertStruct;
	InsertStruct.hParent = hParent;	
	InsertStruct.hInsertAfter = TVI_LAST;
	InsertStruct.itemex.mask = TVIF_IMAGE | TVIF_TEXT | TVIF_CHILDREN | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	InsertStruct.itemex.pszText = sText.GetBuffer(sText.GetLength());  sText.ReleaseBuffer();
	InsertStruct.itemex.iImage = iImage;
	InsertStruct.itemex.cChildren = 1;
	InsertStruct.itemex.lParam = (LPARAM)pNetResource;
	if (iImageSelected == -1)
		InsertStruct.itemex.iSelectedImage = iImage;
	else
		InsertStruct.itemex.iSelectedImage = iImageSelected;
	return m_Tree.InsertItem(&InsertStruct);
}

bool CWallpaperQDlg::PopulateTree(CString sPath, HTREEITEM hParent)
{
	bool bGotChildren = false;						//True if a child is added.	

	//
	//Search for files and populate the CTreeCtrl
	//
	CFileFind finder;
	BOOL bWorking = finder.FindFile(sPath + _T("*.*"));
	while (bWorking)
	{
		bWorking = finder.FindNextFile();
		if (finder.IsDots())
			continue;
		if (finder.IsDirectory())
		{
			InsertItem(hParent, NULL, finder.GetFileName(), DRIVE_NO_ROOT_DIR, DRIVE_UNKNOWN);
			bGotChildren = true;
		}
	}

	//
	//Remove the [+] if no children
	//
	if (!bGotChildren)
	{
		TVITEM item =
		{
			0
		};
		item.mask = TVIF_HANDLE | TVIF_CHILDREN;
		item.hItem = hParent;
		item.cChildren = 0;
		m_Tree.SetItem(&item);
	}
	return bGotChildren;
}

CString CWallpaperQDlg::GetItemPath(HTREEITEM hItem)
{
	CString sRet;
	do
	{
		//End with a share name.
		NETRESOURCE *const pNetResource = (NETRESOURCE*)(m_Tree.GetItemData(hItem));
		if (pNetResource)
		{
			sRet = CString(pNetResource->lpRemoteName) + _T('\\') + sRet; 		
			break;
		}
		//Add the directory name to the path.
		sRet = m_Tree.GetItemText(hItem) + _T('\\') + sRet; 
		hItem = m_Tree.GetParentItem(hItem);
	}
	while (hItem);
	return sRet;
}

void CWallpaperQDlg::OnItemexpandingTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	CWaitCursor CursorWaiting;						//Show the wait cursor while expanding
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	ASSERT(pNMTreeView);
	ASSERT(pResult);
	//Only action 2 notifications
	if (pNMTreeView->action == 2)
	{
		//Update location display
		CString sPath = GetItemPath(pNMTreeView->itemNew.hItem);
		//Refresh children
		if (!m_Tree.GetChildItem(pNMTreeView->itemNew.hItem))
		{
			PopulateTree(sPath, pNMTreeView->itemNew.hItem);
			if (m_Tree.GetSelectedItem() != pNMTreeView->itemNew.hItem)
				m_Tree.SelectItem(pNMTreeView->itemNew.hItem);
		}
	}
	*pResult = 0;
}

void CWallpaperQDlg::OnSelchangedTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	ASSERT(pNMTreeView);
	ASSERT(pResult);
	//Disable search on Workstation roots
	if (m_Tree.GetItemData(pNMTreeView->itemNew.hItem))
	{
		GetDlgItem(IDOK)->EnableWindow(false);
	}
	else
	{
		GetDlgItem(IDOK)->EnableWindow(true);
		cstrBrowsePath = GetItemPath(pNMTreeView->itemNew.hItem);
	}
	*pResult = 0;

	FillList();
	//
	//TODO add code here to process users selection
	//
	//	ASSERT( GetDlgItem( IDC_STATIC_NOTE ) );
	//	SetDlgItemText( IDC_STATIC_NOTE, sPath );
}

BOOL CWallpaperQDlg::DestroyWindow()
{
	//Start at the root deleting.
	HTREEITEM hItemCur = m_Tree.GetRootItem();		//Current item under inspection

	//
	//While their is still data in the tree
	//
	while (hItemCur)
	{
		HTREEITEM hItem = m_Tree.GetChildItem(hItemCur);
		//Has children then make the child current
		if (hItem)
		{
			hItemCur = hItem;
			continue;
		}

		//Item has not children so we shall destroy it.
		//but first we must decide who is to take its place.
		HTREEITEM hNextSibItem = m_Tree.GetNextSiblingItem(hItemCur);
		HTREEITEM hPrevSibItem = m_Tree.GetPrevSiblingItem(hItemCur);
		HTREEITEM hParentItem = m_Tree.GetParentItem(hItemCur);
		//Get item data to check if lparam is to be destroyed
		NETRESOURCE *const pNetResource = (NETRESOURCE*)m_Tree.GetItemData(hItemCur);//(item.lParam);
		if (pNetResource)
		{
			delete[] (pNetResource->lpLocalName);
			delete[] (pNetResource->lpRemoteName);
			delete[] (pNetResource->lpComment);
			delete[] (pNetResource->lpProvider);
			delete pNetResource;
		}
		m_Tree.DeleteItem(hItemCur);
		hItemCur = NULL;
		//Determine which item is next to recieve the focus
		if (hParentItem)
			hItemCur = hParentItem;
		if (hPrevSibItem)
			hItemCur = hPrevSibItem;
		if (hNextSibItem)
			hItemCur = hNextSibItem;
	}	
	//All items removed from list now. Lets trash this place and go home.
	return CDialog::DestroyWindow();
}

void CWallpaperQDlg::OnBnClickedEnablecalendar()
{
	UpdateData(true);
	GetDlgItem(IDC_XPOS)->EnableWindow(m_enableCalendar);
	GetDlgItem(IDC_YPOS)->EnableWindow(m_enableCalendar);
	GetDlgItem(IDC_FONT_BTN)->EnableWindow(m_enableCalendar);
	GetDlgItem(IDC_COLOR_BTN)->EnableWindow(m_enableCalendar);
	GetDlgItem(IDC_TRANSPARENCY)->EnableWindow(m_enableCalendar);
	GetDlgItem(IDC_BACKGROUNDTRANSPARENCY)->EnableWindow(m_enableCalendar);
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnDeltaposSpin1(NMHDR *pNMHDR, LRESULT *pResult)
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnDeltaposSpin2(NMHDR *pNMHDR, LRESULT *pResult)
{
	m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnEnChangeNbrows()
{
	if(IsWindowVisible())
		m_btnApply.EnableWindow();
}

void CWallpaperQDlg::OnEnChangeNbcols()
{
	if(IsWindowVisible())
		m_btnApply.EnableWindow();
}
